dojo.provide("com.ibm.pb.iwidget.PageModeWidgetHandler");

/**
 * This module is used when parsing the page at initialization time to detect
 * iwidgets that are only used in certain page modes and to dynamically turn them 
 * into lazy-loading iwidgets by using a rendering controller like the
 * com.ibm.pb.control.RenderingController with the addModeWidgets function.
 * It's the RenderingController's responsibility to load those iwidgets when
 * the a particular page mode is entered for the first time.
 */
dojo.declare("com.ibm.pb.iwidget.PageModeWidgetHandler", null, {
	
	CLASS_PREFIX: "pb-pageMode-",
	
	renderingController: null,
	matchingModes: [],
	
	constructor: function(args) {
		dojo.safeMixin(this, args);
		if(!this.matchingModes) {
			this.matchingModes = [];
		}
		this.modeWidgets = {};
		dojo.forEach(this.matchingModes, function(mode){
			this.modeWidgets[mode] = [];
		}, this);
	},
	preProcessCallbackFunc: function(domNode, livetextList, tagEntry) {
		if(this.renderingController && this._isMatchingIWidgets(tagEntry)) {
			dojo.forEach(livetextList, function(node){
				for(var mode in this.modeWidgets) {
					if(dojo.hasClass(node, this.CLASS_PREFIX + mode)) {
						// if this inline iwidget declaration specifies
						// a class name with the given CLASS_PREFIX and
						// a mode we're matching, then turn it into a lazy-
						// loaded iwidget for that page mode
						node.setAttribute("lazyLoad", "true");
						var id = node.getAttribute("id");
						this.modeWidgets[mode].push(id);
						this.renderingController.addModeWidgets(mode, [id]);
					}
				}
			}, this);
		}
	},
	postProcessCallbackFunc: function(domNode, livetextList, errors, tagEntry) {
		// no-op
	},
	_isMatchingIWidgets: function(tagEntry) {
		return tagEntry.match && tagEntry.match.indexOf(".iw-iWidget") > -1;
	}
});